//
//  Chromium_ESViewController.m
//  Chromium ES
//
//  Created by Daniel Eisner on 11/5/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "Chromium_ESViewController.h"
#import "EAGLView.h"

#include <ctime>
#include "Config.h"
#include "Global.h"
#include "HiScore.h"
#include "MainGL.h"

#include "HeroAircraft.h"
#include "MenuGL.h"
#include "MainIOS.h"
#include "Audio.h"

// Uniform index.
enum {
    UNIFORM_TRANSLATE,
    NUM_UNIFORMS
};
GLint uniforms[NUM_UNIFORMS];

// Attribute index.
enum {
    ATTRIB_VERTEX,
    ATTRIB_COLOR,
    NUM_ATTRIBUTES
};

@interface Chromium_ESViewController ()
@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) CADisplayLink *displayLink;
@end

@implementation Chromium_ESViewController

@synthesize animating, context, displayLink;

- (void)awakeFromNib
{
    EAGLContext *aContext = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
    
    if (!aContext)
        NSLog(@"Failed to create ES context");
    else if (![EAGLContext setCurrentContext:aContext])
        NSLog(@"Failed to set ES context current");
    
	self.context = aContext;
	[aContext release];
	
    [(EAGLView *)self.view setContext:context];
    [(EAGLView *)self.view setFramebuffer];
        
    animating = FALSE;
    animationFrameInterval = 1;
    self.displayLink = nil;

	is_init = false;
	
	srand(time(NULL));
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	[self touchesMoved:touches withEvent:event];

	// This is basically a mousedown event
	
	Global	*game = Global::getInstance();
	if(game->gameMode == Global::Game) {
		game->hero->fireGun(true);
	}
	else if(game->gameMode == Global::Menu) {
		UITouch *touch = [touches anyObject];
		CGPoint currentPosition = [touch locationInView:self.view];

		NSLog(@"event: (%f, %f)", currentPosition.x, currentPosition.y);
		
		game->menu->mousePress(MainToolkit::Left, currentPosition.x, currentPosition.y);
	}
	else if(game->gameMode == Global::HeroDead) {
		game->gameMode = Global::Menu;
		game->audio->setMusicMode(Audio::MusicMenu);
	}	
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	if (event.type == UIEventTypeTouches) {
		
		UITouch *touch = [touches anyObject];
		CGPoint currentPosition = [touch locationInView:self.view];
		
		Global	*game = Global::getInstance();
		game->hero->moveAbsoluteEvent(currentPosition.x, currentPosition.y);
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	// basically a mouseup
	if (event.type == UIEventTypeTouches) {
		
		UITouch *touch = [touches anyObject];
		CGPoint currentPosition = [touch locationInView:self.view];
		
		Global	*game = Global::getInstance();
		game->hero->moveAbsoluteEvent(currentPosition.x, currentPosition.y);
		
		
		if(game->gameMode == Global::Game) {
			game->hero->fireGun(false);
		}
	}
}

- (void)dealloc
{
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
    
    [context release];
    
    [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self startAnimation];
    
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [self stopAnimation];
    
    [super viewWillDisappear:animated];
}


- (void)viewDidUnload
{	
	[super viewDidUnload];
	
	Global	*game = Global::getInstance();
	Config	*config = Config::instance();
	HiScore	*hiScore = HiScore::getInstance();
	
	//hiScore->destroy();
	//config->destroy();
	//game->destroy();
		
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	self.context = nil;	
}

- (NSInteger)animationFrameInterval
{
    return animationFrameInterval;
}

- (void)setAnimationFrameInterval:(NSInteger)frameInterval
{
    /*
	 Frame interval defines how many display frames must pass between each time the display link fires.
	 The display link will only fire 30 times a second when the frame internal is two on a display that refreshes 60 times a second. The default frame interval setting of one will fire 60 times a second when the display refreshes at 60 times a second. A frame interval setting of less than one results in undefined behavior.
	 */
    if (frameInterval >= 1)
    {
        animationFrameInterval = frameInterval;
        
        if (animating)
        {
            [self stopAnimation];
            [self startAnimation];
        }
    }
}

- (void)startAnimation
{
    if (!animating)
    {
        CADisplayLink *aDisplayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(drawFrame)];
        [aDisplayLink setFrameInterval:animationFrameInterval];
        [aDisplayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        self.displayLink = aDisplayLink;
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
    {
        [self.displayLink invalidate];
        self.displayLink = nil;
        animating = FALSE;
    }
}

- (void)drawFrame
{
	//-- Draw our scene...
	[(EAGLView *)self.view setContext:context];
    [(EAGLView *)self.view setFramebuffer];
	
	if (!is_init) {
		is_init = true;
		Config	*config	= Config::init();
		Global	*game	= Global::init();
		HiScore	*hiScore = HiScore::init();
		game->toolkit = new MainIOS();
		game->generateRandom();
		game->createGame();
	}
	
	Global	*game = Global::getInstance();
	game->mainGL->drawGL();
	
    [(EAGLView *)self.view presentFramebuffer];
	
	//-- Update the scene...
	++frames;
	game->frame++;
	
	if( !(game->gameFrame%10) )
	{
		Config	*config = Config::instance();
		uint now_time = CACurrentMediaTime() * 1000.0;
		if(last_time)
		{
			game->fps = (10.0/(now_time-last_time))*1000.0;
		}
		last_time = now_time;
		
		if(game->gameMode != Global::Menu)
		{
			if(game->gameFrame < 400)
			{
				if(game->fps < 48.0 && game->gameSpeed < 1.0)
				{
					game->gameSpeed += 0.02;
					if( config->debug() ) 
						fprintf(stdout, ("init----> %3.2ffps gameSpeed = %g\n"),
								game->fps, game->gameSpeed);
				}
				else if(game->gameFrame > 20)
				{
					float tmp = 50.0/game->fps;
					tmp = 0.8*targetAdj + 0.2*tmp;
					targetAdj = floor(100.0*(tmp+0.005))/100.0;
					if( config->debug() ) fprintf(stdout, "init----> %3.2ffps targetAdj = %g, tmp = %g\n", game->fps, targetAdj, tmp);
				}
			}
			else if( config->autoSpeed() && (game->fps > 30.0 && game->fps < 100.0))  // discount any wacky fps from pausing
			{
				//game->speedAdj = targetAdj;
				// Everything was originally based on 50fps - attempt to adjust
				// if we're outside a reasonable range
				float tmp = 50.0/game->fps;
				if( fabs(targetAdj-tmp) > 0.1)
				{
					game->speedAdj = tmp;
					if( config->debug() ) fprintf(stdout, "adjust--> %3.2f targetAdj = %g -- game->speedAdj = %g\n", game->fps, targetAdj, game->speedAdj);
				}
				else
					game->speedAdj = targetAdj;
			}
			else
				game->speedAdj = targetAdj;
			
			//				if( !(frames%500) )
			//					if( config->debug() ) fprintf(stdout, _("fps = %g speedAdj = %g\n"), game->fps, game->speedAdj);
		}
		
	}	
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}


@end
