/*
 *  achievements.cpp
 *  Chromium ES
 *
 *  Created by Daniel Eisner on 11/15/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "achievements.h"

#ifdef HAVE_OPENFEINT
#import "OpenFeint.h"
#import "OFAchievement.h"
#endif

/**
 * Static singleton instance
 */
Achievements* Achievements::theAchievements = NULL;

Achievements::Achievements()
{
	loadFromFile();
}

Achievements *Achievements::getInstance()
{
	if (theAchievements == NULL) {
		theAchievements = new Achievements();
	}
	
	return theAchievements;
}

void Achievements::setAchievement(int achievement)
{
#ifdef HAVE_OPENFEINT
	NSString *aId = [[[NSString alloc] initWithFormat:@"%d", achievement] autorelease];
	[[OFAchievement achievement: aId] updateProgressionComplete: 100.0f andShowNotification: YES]; 
#endif
}


char * Achievements::getPlayerName()
{
#ifdef HAVE_OPENFEINT
	static char playerName[512];

		// XXX TODO: get the last logged in user if there is no current user
	OFUser* player = [OpenFeint localUser];
	NSString *playerString = [player name];

	
		//[OpenFeint lastLoggedInUserId]
	
	[playerString getCString:playerName
				   maxLength:512
					encoding:[NSString defaultCStringEncoding]];

	return playerName;
#else
	return (char*)"player";
#endif
}


char * Achievements::getFileName(const char *playerName)
{
	static char pathdir[512];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	
	[documentsDirectory getCString:pathdir
						 maxLength:512
						  encoding:[NSString defaultCStringEncoding]];
	
	strcat(pathdir, "/");
	if (playerName == NULL) {
		strcat(pathdir, getPlayerName());
	}
	else {
		strcat(pathdir, playerName);
	}

	strcat(pathdir, ".record");
	return pathdir;	
}


void Achievements::loadFromFile()
{
	char *fileName = getFileName();
	char *playerName = getPlayerName();
	
	FILE	*file;
	file = fopen(fileName, "r");
	if (file) {
		int len = fread((void*)&data, sizeof(data), 1, file);
		if (len != 1) {
			NSLog(@"Only read %d of %d bytes from %s", len, sizeof(data), fileName);
		}
		if (strcmp(playerName, data.playerName) != 0) {
			NSLog(@"Loaded wrong player achievements (%s) from %s", playerName, fileName);
		}

		fclose(file);
	}
	else {
		// Start off with blank stats
		memset((void*)&data, 0, sizeof(data));
		strcpy(data.playerName, playerName);
		if (strcmp(playerName, data.playerName) != 0) {
			NSLog(@"New player(%s) from %s", playerName, fileName);
		}
		
	}
}


void Achievements::saveToFile()
{
	char * fileName = getFileName(data.playerName);
	
	FILE	*file;
	file = fopen(fileName, "w");
	if (file) {
		int written = fwrite((void*)&data, sizeof(data), 1, file);
		fclose(file);
		
		if (written != 1) {
			NSLog(@"Only wrote %d of %d bytes to %s", written, sizeof(data), fileName);
			perror("Writing to file");
		}
	}
	else {
		NSLog(@"Unable to open %s for writing", fileName);
	}
}


/**
 * If the player has changed, save their achievements and then load
 * the new player's achievements.
 */
void Achievements::checkPlayer()
{
	char * player = getPlayerName();
	if (strcmp(player, data.playerName) != 0) {
		saveToFile();
		loadFromFile();
	}
}


void Achievements::checkAchievements()
{
	if (data.enemiesKilled == ACHIEVE_ENEMY_KILLED_GOAL_1) {
		setAchievement(ID_ENEMY_KILLED_GOAL_1);
	}
	else if (data.enemiesKilled == ACHIEVE_ENEMY_KILLED_GOAL_2) {
		setAchievement(ID_ENEMY_KILLED_GOAL_2);
	}
	else if (data.enemiesKilled == ACHIEVE_ENEMY_KILLED_GOAL_3) {
		setAchievement(ID_ENEMY_KILLED_GOAL_3);
	}
	
	
	fprintf(stderr, "bd %d\n", data.bulldozersKilled);
	fprintf(stderr, "ufo %d\n", data.ufosKilled);
	fprintf(stderr, "tot %d\n", data.enemiesKilled);
	
	if (data.bulldozersKilled == ACHIEVE_BULLDOZER_KILLED_GOAL) {		
		setAchievement(ID_BULLDOZER_KILLED_GOAL);
	}
	
	if (data.ufosKilled == ACHIEVE_UFO_KILLED_GOAL) {
		setAchievement(ID_UFO_KILLED_GOAL);
	}
	
	if (data.ufosKilled == ACHIEVE_GNAT_KILLED_GOAL) {
		setAchievement(ID_GNAT_KILLED_GOAL);
	}
	
	if (data.ufosKilled == ACHIEVE_POWERUP_MISSED_GOAL) {
		setAchievement(ID_POWERUP_MISSED_GOAL);
	}
	
	if (data.ufosKilled == ACHIEVE_EXTRA_LIVES_GOAL) {
		setAchievement(ID_EXTRA_LIVES_GOAL);
	}
	
}

