/*
 *  ios_util.cpp
 *  chromium
 *
 *  Created by Daniel Eisner on 10/31/10.
 *  Copyright 2010 ThroughPut Inc. All rights reserved.
 *
 */

#import <CoreFoundation/CoreFoundation.h>
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#import <OpenGLES/EAGLDrawable.h>
#import <OpenGLES/EAGL.h>
#import <OpenGLES/ES1/glext.h>


#include "ios_util.h"
#include "OpenALSupport.h"
#include "Texture2D.h"

#ifdef HAVE_OPENFEINT
#import "OpenFeint.h"
#import "OFLeaderboard.h"
#endif

void getScrenDim(int *x, int *y) {
	CGRect	rect = [[UIScreen mainScreen] bounds];
	*x = rect.size.width;
	*y = rect.size.height;
}

GLuint iosLoadTex(const char *img_file,
				  const GLint wrapst, const GLint minfilter, const GLint magfilter)
{
	NSString *texfile = [[[NSString alloc] initWithCString:img_file
												encoding:[NSString defaultCStringEncoding]]
						autorelease];


	// Intentional MEMORY LEAK. On iPhone, graphics memory is actually main memory. Releasing this
	// buffer will prevent textures from rendering in opengl. Since we don't de-alloc / re-alloc
	// textures, it's fine to leak here.
	Texture2D *tex = [[Texture2D alloc] initWithImagePath:texfile sizeToFit:NO
											  pixelFormat:kTexture2DPixelFormat_Automatic];
		
	
	
	glBindTexture(GL_TEXTURE_2D, [tex name]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrapst);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapst);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magfilter);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minfilter);
	
	//NSLog(@"IOS LOADING TEXTURE %d: '%@'", [tex name], texfile);
	
	return [tex name];
}


char* path_to_resources(const char* resource_file, const char* resource_type, char* buffer)
{
	char *fptr = strrchr(resource_file, '/');
	if (fptr != NULL) {
		resource_file = fptr+1;
	}
	
	NSString *fname = [[[NSString alloc] initWithCString:resource_file
												encoding:[NSString defaultCStringEncoding]]
					   autorelease];
	NSString *ftype = [[[NSString alloc] initWithCString:resource_type
												encoding:[NSString defaultCStringEncoding]]
					   autorelease];
					   
	NSString *path = [[NSBundle mainBundle] pathForResource:fname ofType:ftype];
	
	if (path == NULL) {
		NSLog(@"Unable to determine path for %@ %@", fname, ftype);
		return NULL;
	}
	else {
		BOOL retval = [path getCString:buffer maxLength:256 encoding:[NSString defaultCStringEncoding]];
		if (retval == NO) {
			return NULL;
		}
		else {
			return buffer;
		}
	}
}

char* path_to_resource_dir(char* buffer) {
	NSString *resourcePath = [[NSBundle mainBundle] resourcePath];
	//NSLog(@"Resource Path: %@", resourcePath);
	
	BOOL retval = [resourcePath getCString:buffer maxLength:256 encoding:[NSString defaultCStringEncoding]];

	if (retval == NO) {
		return NULL;
	}
	else {
		return buffer;
	}	
}

void checkFrameBuffer() {
	if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
		NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
	}
}


ALuint iosLoadSound(const char * file)
{
    ALenum  error = AL_NO_ERROR;
    ALenum  format;
    ALsizei size;
    ALsizei freq;
    ALuint  buffer;
	
	void*                   data; // XXX: We intentionally leak the audio buffer.
	
		// get some audio data from a wave file
	NSString *path = [[[NSString alloc] initWithCString: file] autorelease];
    CFURLRef fileURL = (CFURLRef)[[NSURL fileURLWithPath:path] retain];
    
    if (fileURL)
    {   
        data = MyGetOpenALAudioData(fileURL, &size, &format, &freq);
        CFRelease(fileURL);
        
        if((error = alGetError()) != AL_NO_ERROR) {
            NSLog(@"error loading sound: %x\n", error);
            exit(1);
        }
        
		alGenBuffers(1, &buffer);
		if((error = alGetError()) != AL_NO_ERROR) {
			NSLog(@"Error Generating Buffers: %x", error);
		}
		
		// use the static buffer data API
        alBufferDataStaticProc(buffer, format, data, size, freq);
        
        if((error = alGetError()) != AL_NO_ERROR) {
            NSLog(@"error attaching audio to buffer: %x\n", error);
        }       
    }
    else {
        NSLog(@"Could not find file!\n");
	}
	
	return buffer;
}

void iosInitSound()
{
	init_audio();
}

char* get_score_file()
{
	static char pathdir[512];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	
	[documentsDirectory getCString:pathdir
						maxLength:512
						  encoding:[NSString defaultCStringEncoding]];

	strcat(pathdir, "/hiscore.list");
	return pathdir;
}

char* get_config_file()
{
	static char pathdir[512];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	
	[documentsDirectory getCString:pathdir
						 maxLength:512
						  encoding:[NSString defaultCStringEncoding]];
	
	strcat(pathdir, "/config.settings");
	return pathdir;
}

char* ios_get_player()
{
#ifdef HAVE_OPENFEINT
	static char playerName[512];
	
	OFUser* player = [OpenFeint localUser];
	NSString *playerString = [player name];

	[playerString getCString:playerName
						 maxLength:512
						  encoding:[NSString defaultCStringEncoding]];
	
	return playerName;
#else
	return (char*)"player";
#endif
}

#ifdef HAVE_OPENFEINT
void activate_openfeint()
{
	[OpenFeint launchDashboard];
}


void openfeint_submit_score(int skill, int score)
{
	OFHighScore* newHighScore = [[[OFHighScore alloc] initForSubmissionWithScore: score] autorelease];
	newHighScore.displayText = nil;
	newHighScore.customData = nil;

	NSString* leaderboards[] = {
		@"55XXXX",  // fish
		@"55XXXX",  // whimp
		@"55XXXX",  // easy
		@"55XXXX",  // normal
		@"55XXXX",  // experienced
		@"55XXXX",  // fun
		@"55XXXX",  // insane
		@"55XXXX",  // impossible
	};
	
	NSString *leaderboard_id = leaderboards[0];
	if (skill > 1 && skill < 10) {
		leaderboard_id = leaderboards[int(skill - 2)];
	}
	
	[newHighScore submitTo:[OFLeaderboard leaderboard:leaderboard_id]];
	
	
}
#endif


